# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('local_sendgrid', '0003_auto_20170608_2155'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transactionalemail',
            name='template_id',
            field=models.CharField(max_length=255),
        ),
    ]



import json
from django.test import Client, TestCase
from .models import Contact, ContactList, Campaign, Click, Open, Delivered, TransactionalEmail
from datetime import date


# Create your tests here.

import logging


class ContactTests(TestCase):
    def test_sengrid_check(self):
        contact = Contact.objects.create(email="asdasd+789@asdf.cl")
        contact.sendgrid_delete()
        self.assertFalse(contact.sendgrid_check())

        contact.sendgrid_id = u"asd123sd"
        contact.save()
        self.assertFalse(contact.sendgrid_check())

        contact.sendgrid_id = contact.email.encode('base64')
        contact.save()
        contact.sendgrid_upload()
        self.assertTrue(contact.sendgrid_check())

        self.clean_up()

    def test_sendgrid_upload(self):
        contact = Contact.objects.create(email="asdasd@asdf.cl")
        # Contact.objects.get(email="asdasd@asdf.cl")
        contact.sendgrid_upload()

        self.assertTrue(contact.sendgrid_check())

        contact.sendgrid_delete()

    def test_sendgrid_delete(self):
        contact = Contact.objects.create(email="asdasd@asdf.cl")
        contact.sendgrid_upload()

        contact.sendgrid_delete()

        self.assertFalse(contact.sendgrid_check())

        self.clean_up()

    def test_sendgrid_upload_queryset(self):
        count = Contact.objects.all().sendgrid_upload().count()
        self.assertEqual(0, count, "If a don't have Contacts I can't updated them")

        Contact.objects.create(email="asd")

        contacts = Contact.objects.all()

        contacts = contacts.sendgrid_upload()

        self.assertEqual(
            1,
            len([contact for contact in contacts if not contact.sendgrid_check()])
        )

        self.assertEqual(
            0,
            len([contact for contact in contacts if contact.sendgrid_check()])
        )

        Contact.objects.create(email="asaavedra@masaval.cl")
        Contact.objects.create(email="asaavedra+123@masaval.cl")
        Contact.objects.create(email="asdasdasd")

        contacts = Contact.objects.all()

        contacts = contacts.sendgrid_upload()

        self.assertEqual(
            2,
            len([contact for contact in contacts if not contact.sendgrid_check()])
        )

        self.assertEqual(
            2,
            len([contact for contact in contacts if contact.sendgrid_check()])
        )

        # clean

        self.clean_up()

    def test_big_bulk_contact_creation(self):
        for index in range(20):
            Contact.objects.create(email="asdf{}@asdf.asd".format(index))

        contacts = Contact.objects.all()

        contacts = contacts.sendgrid_upload()

        self.assertEqual(20, contacts.exclude(sendgrid_id=None).count())
        for valid in contacts:
            self.assertTrue(
                valid.sendgrid_check(),
                "{} marked as wrong sendgrid id!".format(valid.email)
            )
        self.clean_up()

    @staticmethod
    def clean_up():
        for contact in Contact.objects.all():
            contact.sendgrid_delete()


class ContactListTest(TestCase):
    def setUp(self):
        self.contact_list = ContactList.objects.create()

    def tearDown(self):
        [contact.sendgrid_delete() for contact in self.contact_list.contact_set.all()]
        self.contact_list.sendgrid_delete()

    def test_creation_name(self):
        self.assertEqual(self.contact_list.name, 'ContactList-{}'.format(date.today()))

    def test_sendgrid_check(self):
        self.assertFalse(self.contact_list.sendgrid_check())

    def test_sendgrid_upload(self):
        self.contact_list.sendgrid_upload()
        self.assertTrue(self.contact_list.sendgrid_check())

    def test_sendgrid_contact_upload(self):
        contacts = [
            Contact.objects.create(email="asdf{}@asdf.asd".format(index))
            for index in range(20)
        ]
        self.contact_list.contact_set = contacts
        self.contact_list.sendgrid_upload()
        self.assertTrue(self.contact_list.sendgrid_check())

        self.contact_list.sendgrid_contact_upload()


class CampaignTest(TestCase):
    def setUp(self):
        self.uploaded_campaign = Campaign.objects.create(
            title='test',
            subject='test subject',
            html_content="<html></html>"
        )
        self.contact_list = ContactList.objects.create()
        self.campaign = Campaign.objects.create(
            contact_list=self.contact_list)

    def tearDown(self):
        self.uploaded_campaign.sendgrid_delete()
        if self.uploaded_campaign.contact_list:
            self.uploaded_campaign.contact_list.sendgrid_delete()
            for contact in self.uploaded_campaign.contact_list.contact_set.all():
                contact.sendgrid_delete()

        if self.campaign.contact_list:
            for contact in self.campaign.contact_list.contact_set.all():
                contact.sendgrid_delete()
            self.campaign.sendgrid_delete()
            self.campaign.contact_list.sendgrid_delete()

    def test_title(self):
        campaign = Campaign.objects.create()
        self.assertEqual(campaign.title, "Campaign-{}".format(date.today()))

    def test_sendgrid_check(self):
        self.uploaded_campaign.sendgrid_upload()
        self.assertFalse(self.campaign.sendgrid_check())
        self.assertTrue(self.uploaded_campaign.sendgrid_check())

    def test_create_from_sendgrid_id(self):
        self.uploaded_campaign.sendgrid_upload()
        sendgrid_id = self.uploaded_campaign.sendgrid_id
        campaign = Campaign.create_from_sendgrid_id(sendgrid_id)
        self.assertEqual(campaign.title, 'test')
        self.assertEqual(campaign.subject, 'test subject')

    def test_sendgrid_data(self):
        self.uploaded_campaign.sendgrid_upload()
        data = self.uploaded_campaign.sendgrid_data()
        logging.debug(data)
        self.assertEqual(len(data['list_ids']), 0)

    def test_sendgrid_upload(self):
        campaign = self.uploaded_campaign
        campaign.contact_list = self.contact_list
        contacts = [
            Contact.objects.create(email="asdf{}@asdf.asd".format(index))
            for index in range(20)
        ]
        campaign.contact_list.contact_set = contacts
        campaign.contact_list.sendgrid_contact_upload()
        campaign.contact_list.sendgrid_upload()
        campaign.sendgrid_upload()
        self.assertTrue(campaign.sendgrid_id is not None)


class EventTest(TestCase):
    def setUp(self):
        self.email = 'asdasd+789@asdf.cl'
        self.data = {
            'email': self.email,
            'timestamp': '145000',
            'sg_message_id': '1231ss',
            'category': 'asdasd',
            'url': 'http://asdasddasd/',
            'ip': '192.168.1.1',
            'useragent': 'EvilCorp',
            'smtp-id': 'asd123'
        }

        self.contact = Contact.objects.create(email=self.email)

    def test_click_factory(self):
        click = Click.factory(self.data)

        self.assertEqual(click.email, self.email)
        self.assertEqual(click.contact, self.contact)
        self.assertEqual(click.category, self.data['category'])

    def test_open_factory(self):
        open_event = Open.factory(self.data)

        self.assertEqual(open_event.email, self.email)
        self.assertEqual(open_event.contact, self.contact)
        self.assertEqual(open_event.ip, self.data['ip'])
        self.assertEqual(open_event.useragent, self.data['useragent'])

    def test_delivered_factory(self):
        delivered = Delivered.factory(self.data)

        self.assertEqual(delivered.email, self.email)
        self.assertEqual(delivered.contact, self.contact)
        self.assertEqual(delivered.smtp_id, self.data['smtp-id'])


class TransactionalEmailTest(TestCase):
    def test_sendgrid_send(self):
        contact = Contact.objects.create(email="asdf@asdf.asd")
        email = TransactionalEmail.objects.create(recipient=contact, template_id=1)
        self.assertTrue(email.sendgrid_send(sandbox=True))


class ViewTest(TestCase):
    def setUp(self):
        self.email = 'asdasd+789@asdf.cl'
        self.contact = Contact.objects.create(email=self.email)
        self.data = [{
            'event': 'click',
            'email': self.email,
            'timestamp': '145000',
            'sg_message_id': '1231ss',
            'category': 'asdasd',
            'url': 'http://asdasddasd/',
            'ip': '192.168.1.1',
            'useragent': 'EvilCorp',
            'smtp-id': 'asd123'
        }, {
            'event': 'open',
            'email': self.email,
            'timestamp': '145000',
            'sg_message_id': '1231ss',
            'category': 'asdasd',
            'url': 'http://asdasddasd/',
            'ip': '192.168.1.1',
            'useragent': 'EvilCorp',
            'smtp-id': 'asd123'
        }, {
            'event': 'delivered',
            'email': self.email,
            'timestamp': '145000',
            'sg_message_id': '1231ss',
            'category': 'asdasd',
            'url': 'http://asdasddasd/',
            'ip': '192.168.1.1',
            'useragent': 'EvilCorp',
            'smtp-id': 'asd123'
        }, {
            'event': 'NonExistan',
            'email': self.email,
            'timestamp': '145000',
            'sg_message_id': '1231ss',
            'category': 'asdasd',
            'url': 'http://asdasddasd/',
            'ip': '192.168.1.1',
            'useragent': 'EvilCorp',
            'smtp-id': 'asd123'
        }, {
            'event': 'click',
            'email': 'asdasd+123123@asdf.cl',
            'timestamp': '145000',
            'sg_message_id': '1231ss',
            'category': 'asdasd',
            'url': 'http://asdasddasd/',
            'ip': '192.168.1.1',
            'useragent': 'EvilCorp',
            'smtp-id': 'asd123'
        }]

    def test_sendgrid_event(self):
        c = Client()
        self.data[0]['event'] = 'click'
        response = c.post(
            '/sendgrid/events/',
            data=json.dumps(self.data),
            content_type="application/json")
        self.assertEqual(response.status_code, 201)



# -*- encoding: utf-8 -*-
import logging
import json

from django.db import models

from django.conf import settings
from urllib2 import HTTPError


from datetime import date

# sendgrid
from sendgrid.helpers.mail import Mail, Email, OpenTracking, TrackingSettings, ClickTracking
from sendgrid.helpers.mail import Personalization, MailSettings, SandBoxMode
import sendgrid


sendgrid_api_client = sendgrid.SendGridAPIClient(
    apikey=settings.SENDGRID_API_KEY
).client


class ContactSendgridQuerySet(models.QuerySet):

    def sendgrid_upload(self):
        """
        Upload and up updates de contact Queryset to senddgrid
        Returns:
            QuerySet: A querySet of Contacts
        """
        request_body = [contact.email_data() for contact in self]
        response = sendgrid_api_client.contactdb.recipients.post(
            request_body=request_body
        )

        response = json.loads(response.body)

        for contact in self.exclude(sendgrid_id__in=response['persisted_recipients']):
            logging.error("Correo {} no valido".format(contact.email))

        return self.all()


class ContactList(models.Model):

    name = models.CharField(
        max_length=255,
    )

    sendgrid_id = models.CharField(
        max_length=255,
        unique=True,
        db_index=True,
        editable=False,
        null=True
    )

    def __unicode__(self):
        return u"{}".format(self.name)

    def save(self, *args, **kwargs):
        if self.name is u'':
            self.name = u"{}-{}".format(type(self).__name__, date.today())
        super(ContactList, self).save(*args, **kwargs)

    def contact_filter(self):
        """Filter the contacts that are going to be uploaded.

        Returns:
            QuerySet: Only the valid contacts
        """
        return self.contact_set

    def sendgrid_upload(self):
        if not self.sendgrid_check():
            try:
                response = sendgrid_api_client.contactdb.lists.post(
                    request_body={'name': self.name}
                )
                response = json.loads(response.body)
                self.sendgrid_id = response['id']
                self.save()
            except HTTPError as e:
                if e.code == 400:
                    logging.error('List all ready exists')
                else:
                    logging.error(e)
                raise e

    def sendgrid_check(self):
        if self.sendgrid_id:
            response = sendgrid_api_client.contactdb.lists._(self.sendgrid_id).get()
            response = json.loads(response.body)
            logging.debug(response)
            return self.name == response['name']
        return False

    def sendgrid_delete(self):
        if self.sendgrid_check():
            sendgrid_api_client.contactdb.lists._(self.sendgrid_id).delete()
            self.sendgrid_id = None
            self.save()

    def sendgrid_contact_upload(self):
        if self.contact_set.count() > 0:
            self.contact_set.sendgrid_upload()
            request_body = [contact.sendgrid_id for contact in self.contact_filter().all()]
            try:
                sendgrid_api_client.contactdb.lists._(self.sendgrid_id).recipients.post(
                    request_body=request_body
                )
            except HTTPError as e:
                logging.error(e)


class Contact(models.Model):
    """
    Base model for contact

    Attributes:
        contact_lists (TYPE): Description
        email (TYPE): Description
        objects (TYPE): Description
        sendgrid_id (CharField): should be the email encoded in base64
    """
    email = models.EmailField(max_length=255)
    contact_lists = models.ManyToManyField(ContactList)

    # This one should be the email encoded in base64
    sendgrid_id = models.CharField(
        max_length=255,
        unique=True,
        db_index=True,
        editable=False
    )

    objects = ContactSendgridQuerySet.as_manager()

    def __unicode__(self):
        return u"{} {}".format(self.email, self.sendgrid_id)

    def save(self, *args, **kwargs):
        self.email = self.email.lower()
        self.sendgrid_id = self.email.encode('base64')
        super(Contact, self).save(*args, **kwargs)

    def email_data(self):
        today = date.today().strftime('%s')

        return {
            'email': self.email,
            'upload_date': today,
        }

    def sendgrid_upload(self):
        type(self).objects.filter(id=self.id)\
            .sendgrid_upload()
        self.refresh_from_db()

    def sendgrid_check(self):
        logging.debug("{}:{}".format(self.email, self.sendgrid_id))
        if self.sendgrid_id == self.email.encode('base64'):
            try:
                response = sendgrid_api_client.contactdb.recipients._(self.sendgrid_id).get()
                response = json.loads(response.body)
                logging.debug("{} == {}".format(response['email'], self.email))
                return response['email'].lower() == self.email.lower()
            except HTTPError as e:
                logging.debug(e)
                if e.code == 400:
                    return False
                if e.code == 404:
                    return False
                else:
                    raise(e)
        return False

    def sendgrid_delete(self):
        if self.sendgrid_check():
            sendgrid_id = self.sendgrid_id
            try:
                sendgrid_api_client.contactdb.recipients._(sendgrid_id).delete()
                return sendgrid_id
            except HTTPError as e:
                logging.debug(e)
                if e.code == 400:
                    return sendgrid_id
                if e.code == 404:
                    return sendgrid_id
                else:
                    raise(e)


class Campaign(models.Model):

    template = models.IntegerField(null=True)

    title = models.CharField(
        max_length=100,
    )

    subject = models.CharField(
        max_length=500,
    )

    sender = models.IntegerField(null=True)

    html_content = models.TextField()

    plain_content = models.TextField()

    sendgrid_id = models.CharField(
        max_length=255,
        unique=True,
        db_index=True,
        editable=False,
        null=True
    )

    contact_list = models.ForeignKey(ContactList, null=True)

    def __unicode__(self):
        return u"{}".format(self.title)

    def save(self, *args, **kwargs):
        if self.title is u'':
            self.title = u"{}-{}".format(type(self).__name__, date.today())
        super(Campaign, self).save(*args, **kwargs)

    def sendgrid_check(self):
        if self.sendgrid_id:
            response = sendgrid_api_client.campaigns._(self.sendgrid_id).get()
            response = json.loads(response.body)
            return self.sendgrid_id == response['id']
        return False

    def sendgrid_data(self):
        if self.sendgrid_id:
            response = sendgrid_api_client.campaigns._(self.sendgrid_id).get()
            response = json.loads(response.body)
            return response
        return None

    def sendgrid_upload(self):
        if not self.sendgrid_check():
            campaign_data = {
                'title': self.title,
                'subject': self.subject,
                'sender': self.sender,
                'html_content': self.html_content,
                'plain_content': self.plain_content,
                'list_ids': []
            }
            if self.contact_list:
                campaign_data['list_ids'].append(self.contact_list.sendgrid_id)
            response = sendgrid_api_client.campaigns.post(
                request_body=campaign_data
            )
            response = json.loads(response.body)
            logging.debug(response)
            self.sendgrid_id = response['id']

    def sendgrid_send(self):
        if self.sendgrid_check():
            sendgrid_api_client.campaigns._(self.sendgrid_id).schedules.now.post()

    def sendgrid_delete(self):
        if self.sendgrid_check():
            sendgrid_api_client.campaigns._(self.sendgrid_id).delete()

    @classmethod
    def create_from_sendgrid_id(cls, sendgrid_id):
        response = sendgrid_api_client.campaigns._(sendgrid_id).get()
        response = json.loads(response.body)
        return cls.objects.create(
            template=sendgrid_id,
            title=response['title'],
            subject=response['subject'],
            sender=response['sender_id'],
            html_content=response['html_content'],
            plain_content=response['plain_content']
        )


class TransactionalEmail(models.Model):

    html_content = models.TextField()
    plain_content = models.TextField()
    template_id = models.CharField(max_length=255)
    recipient = models.ForeignKey(Contact)
    category = models.CharField(max_length=255)

    def email_data(self, sandbox=False):
        mail = Mail()

        mail.from_email = Email("no-reply@masaval.cl", "Masaval")

        personalization = Personalization()
        personalization.add_to(Email(self.recipient.email, "recipient.email"))
        mail.add_personalization(personalization)

        mail_settings = MailSettings()
        mail_settings.sandbox_mode = SandBoxMode(sandbox)
        mail.mail_settings = mail_settings

        mail.template_id = "13b8f94f-bcae-4ec6-b752-70d6cb59f932"
        tracking_settings = TrackingSettings()
        tracking_settings.click_tracking = ClickTracking(True, True)
        tracking_settings.open_tracking = OpenTracking(True)
        mail.tracking_settings = tracking_settings

        return mail

    def sendgrid_send(self, sandbox=False):
        try:
            response = sendgrid_api_client.mail.send.post(
                request_body=self.email_data(sandbox).get()
            )
            return True
        except HTTPError as e:
            logging.error(e)
            return False



import json
from django.views.decorators.csrf import csrf_exempt
from django.core.exceptions import ObjectDoesNotExist
from .models import Event
from django.http import HttpResponse
import logging
# Create your views here.


@csrf_exempt
def events(request):
    """
    Recives a bash of sendgrid events.

    Args:
        request (Request): an http request of django

    Returns:
        HttpResponse: Usign the corresponding code.
    """
    if request.method == 'POST':
        events = json.loads(request.body.decode('utf-8'))
        logging.debug(events)
        for event_data in events:
            # This is a abstract model that return the corresponding event created.
            try:
                Event.factory(event_data)
            except ObjectDoesNotExist:
                logging.warning("{} does not exist in contacts".format(
                    event_data['email']
                ))

        return HttpResponse(status=201)
    return HttpResponse(status=400)



# -*- encoding: utf-8 -*-
from django.db import models
from .models import Contact


class Event(models.Model):

    contact = models.ForeignKey(
        Contact,
        null=True,
        on_delete=models.SET_NULL
    )

    email = models.EmailField(
    )

    timestamp = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )
    sg_message_id = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True

    def __unicode__(self):
        return u'sg_message_id: {}, email: {}, category: {}.'.format(
            self.sg_message_id, self.email, self.category)

    def save(self, *args, **kwargs):
        if self.contact is None:
            self.contact = Contact.objects.get(email__iexact=self.email)
        super(Event, self).save(*args, **kwargs)

    @classmethod
    def factory(cls, data):
        """This is a factory that returns the instance that is required.

        Args:
            data (dict): A dictionary that includes the event,
                         and the data for creating it

        Returns:
            [Click(), Open(), Delivered()]: a instance of one of this class
        """
        classes = {
            'click': Click,
            'open': Open,
            'delivered': Delivered
        }

        event_class = classes.get(data['event'], None)
        if event_class:
            return event_class.factory(data)
        return None


class Click(Event):

    url = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )

    category = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )

    @classmethod
    def factory(cls, data):
        return cls.objects.create(
            email=data['email'],
            timestamp=data['timestamp'],
            sg_message_id=data['sg_message_id'],
            category=data.get('category'),
            url=data['url'],
        )


class Open(Event):
    # requiered fields

    ip = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )
    useragent = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )

    @classmethod
    def factory(cls, data):
        return cls.objects.create(
            email=data['email'],
            timestamp=data['timestamp'],
            sg_message_id=data['sg_message_id'],
            ip=data['ip'],
            useragent=data['useragent'],
        )


class Delivered(Event):
    # requiered fields

    smtp_id = models.CharField(
        max_length=255,
        blank=True,
        null=True,
    )

    @classmethod
    def factory(cls, data):
        return cls.objects.create(
            email=data['email'],
            timestamp=data['timestamp'],
            sg_message_id=data['sg_message_id'],
            smtp_id=data['smtp-id'],
        )



# -*- encoding: utf-8 -*-

from .models import Campaign # noqa
from .models import Contact # noqa
from .models import ContactList # noqa
from .models import TransactionalEmail #noqa
from .events import * # noqa


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('local_sendgrid', '0002_transactional'),
    ]

    operations = [
        migrations.CreateModel(
            name='TransactionalEmail',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('html_content', models.TextField()),
                ('plain_content', models.TextField()),
                ('template_id', models.IntegerField()),
                ('category', models.CharField(max_length=255)),
                ('recipient', models.ForeignKey(to='local_sendgrid.Contact')),
            ],
        ),
        migrations.RemoveField(
            model_name='transactional',
            name='recipient',
        ),
        migrations.DeleteModel(
            name='Transactional',
        ),
    ]



""" this document defines the cases app urls """
from django.conf.urls import patterns, url


urlpatterns = patterns(
    'local_sendgrid.views',
    url(
        r'^events/$',
        'events',
        name='django_sendgrid_event'
    )
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Campaign',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('template', models.IntegerField(null=True)),
                ('title', models.CharField(max_length=100)),
                ('subject', models.CharField(max_length=500)),
                ('sender', models.IntegerField(null=True)),
                ('html_content', models.TextField()),
                ('plain_content', models.TextField()),
                ('sendgrid_id', models.CharField(max_length=255, unique=True, null=True, editable=False, db_index=True)),
            ],
        ),
        migrations.CreateModel(
            name='Click',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(max_length=254)),
                ('timestamp', models.CharField(max_length=255, null=True, blank=True)),
                ('sg_message_id', models.CharField(max_length=255, null=True, blank=True)),
                ('url', models.CharField(max_length=255, null=True, blank=True)),
                ('category', models.CharField(max_length=255, null=True, blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(max_length=255)),
                ('sendgrid_id', models.CharField(unique=True, max_length=255, editable=False, db_index=True)),
            ],
        ),
        migrations.CreateModel(
            name='ContactList',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255)),
                ('sendgrid_id', models.CharField(max_length=255, unique=True, null=True, editable=False, db_index=True)),
            ],
        ),
        migrations.CreateModel(
            name='Delivered',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(max_length=254)),
                ('timestamp', models.CharField(max_length=255, null=True, blank=True)),
                ('sg_message_id', models.CharField(max_length=255, null=True, blank=True)),
                ('smtp_id', models.CharField(max_length=255, null=True, blank=True)),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to='local_sendgrid.Contact', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Open',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(max_length=254)),
                ('timestamp', models.CharField(max_length=255, null=True, blank=True)),
                ('sg_message_id', models.CharField(max_length=255, null=True, blank=True)),
                ('ip', models.CharField(max_length=255, null=True, blank=True)),
                ('useragent', models.CharField(max_length=255, null=True, blank=True)),
                ('contact', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to='local_sendgrid.Contact', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='contact',
            name='contact_lists',
            field=models.ManyToManyField(to='local_sendgrid.ContactList'),
        ),
        migrations.AddField(
            model_name='click',
            name='contact',
            field=models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to='local_sendgrid.Contact', null=True),
        ),
        migrations.AddField(
            model_name='campaign',
            name='contact_list',
            field=models.ForeignKey(to='local_sendgrid.ContactList', null=True),
        ),
    ]
